#include "../include/Server.h"
#include <iostream>

using namespace boost::asio;
using ip::tcp;

/***************
 * con_handler *
 ***************/

con_handler::con_handler(
    basic_socket_acceptor<ip::tcp>::executor_type &io_service)
    : sock(io_service) {}

con_handler::~con_handler() {}

con_handler::pointer
con_handler::create(basic_socket_acceptor<ip::tcp>::executor_type &io_service) {
  return pointer(new con_handler(io_service));
}

tcp::socket &con_handler::socket() { return sock; }

void con_handler::start() {
  sock.async_read_some(buffer(data, max_length),
                       boost::bind(&con_handler::handle_read,
                                   shared_from_this(), placeholders::error,
                                   placeholders::bytes_transferred));

  sock.async_write_some(buffer(message, max_length),
                        boost::bind(&con_handler::handle_write,
                                    shared_from_this(), placeholders::error,
                                    placeholders::bytes_transferred));
}

void con_handler::handle_read(const boost::system::error_code &err,
                              size_t bytes_transferred) {
  if (!err) {
    std::cout << this->data << std::endl;
  } else {
    std::cerr << "error: " << err.message() << std::endl;
    sock.close();
  }
}

void con_handler::handle_write(const boost::system::error_code &err,
                               size_t bytes_transferred) {
  if (!err) {
    std::cout << "Hello World!" << std::endl;
  } else {
    std::cerr << "error: " << err.message() << std::endl;
    sock.close();
  }
}

/**********
 * Server *
 **********/

void Server::start_accept() {
  auto executor = acceptor_.get_executor();
  con_handler::pointer connection = con_handler::create(executor);
  acceptor_.async_accept(connection->socket(),
                         boost::bind(&Server::handle_accept, this, connection,
                                     placeholders::error));
}

Server::Server(io_service &io_service)
    : acceptor_(io_service, tcp::endpoint(tcp::v4(), 1234)) {
  start_accept();
}

Server::~Server() {}

void Server::handle_accept(con_handler::pointer connection,
                           const boost::system::error_code &err) {
  if (!err) {
    connection->start();
  }
  start_accept();
}
